unit MainDataModule;

interface

uses
  System.SysUtils,
  System.Classes,
  JS,
  Web,
  WEBLib.Modules,
  WEBLib.Forms,
  XData.Web.Connection,
  WEBLib.REST,
  WEBLib.Sentry, XData.Web.Client;

type
  TOnClearDashboard = procedure of object;

  TMainData = class(TWebDataModule)
    DataConnection: TXDataWebConnection;
    WebClient: TXDataWebClient;
    AuthConnection: TXDataWebConnection;
    AuthClient: TXDataWebClient;
    WebSentry1: TWebSentry;
    procedure WebDataModuleCreate(Sender: TObject);
  private
    { Private declarations }

    FOnAuthUpdatedProc: TNotifyEvent;
    FLastError: TApplicationError;
    FLastErrorTime: TDateTime;
    FErrorCount: Integer;

    FShops: TStrings;
    FCompanyYearEnd: string;

    FOnClearDashboard: TOnClearDashboard;
    FOnUserChanged: TNotifyEvent;

  public
    procedure OnApplicationError(Sender: TObject; AError: TApplicationError; var Handled: Boolean);

    procedure ClearDashboard;

    [async]
    function ShopLookups: TStrings;
    [async]
    function CompanyYearEnd: string;

    property OnAuthUpdatedProc: TNotifyEvent read FOnAuthUpdatedProc write FOnAuthUpdatedProc;
    property OnClearDashboard: TOnClearDashboard read FOnClearDashboard write FOnClearDashboard;
    property OnUserChanged: TNotifyEvent read FOnUserChanged write FOnUserChanged;

  end;

var
  MainData: TMainData;

implementation

uses
  System.DateUtils,
  WEBLib.Dialogs;

{%CLASSGROUP 'Vcl.Controls.TControl'}
{$R *.dfm}

const
  WEB_SENTRY_DSN = 'https://e57de7e84cce44d89c4fe356cc2341e0@o4504537489473536.ingest.sentry.io/4504537492946944';

procedure TMainData.WebDataModuleCreate(Sender: TObject);
begin
  WebSentry1.DSN := WEB_SENTRY_DSN;
{$IFDEF RELEASE}
  WebSentry1.Release := 'Tenovus-Live@' + Application.Version;
{$ELSE}
  WebSentry1.Release := 'Tenovus-Dev@' + Application.Version;
{$ENDIF}
  WebSentry1.Init;

  FormatSettings.DateSeparator := '/';
  FormatSettings.ShortDateFormat := 'dd/mm/yyyy';
  Application.ErrorType := aeSilent; // aeSilent, aeDialog, aeAlert, aeFooter
  Application.OnError := self.OnApplicationError;

end;

procedure TMainData.ClearDashboard;
begin
  if Assigned(FOnClearDashboard) then
    FOnClearDashboard();
end;

function TMainData.CompanyYearEnd: string;
const
  CSvc = 'IGAUtilsService.GetLookup';
var
  lRetval: TXDataClientResponse;
  lObj: JS.TJSObject;
begin
  if FCompanyYearEnd = '' then
  begin
    lRetval := Await(TXDataClientResponse, WebClient.RawInvokeAsync(CSvc, ['COMPANY-YEAR-END']));
    lObj := JS.toObject(lRetval.ResultAsObject['value']);
    if lObj <> nil then
    begin
      FCompanyYearEnd := JS.ToString(lObj.Properties['Value']);
    end;
  end;
  Result := FCompanyYearEnd;
end;

procedure TMainData.OnApplicationError(Sender: TObject; AError: TApplicationError; var Handled: Boolean);
begin

  FLastErrorTime := Now;

  if (AError.ALineNumber = FLastError.ALineNumber) and (SecondsBetween(Now, FLastErrorTime) < 60) then
  begin
    Inc(FErrorCount);
  end
  else
  begin
    FLastError.ALineNumber := AError.ALineNumber;
    FErrorCount := 1;
  end;

end;

function TMainData.ShopLookups: TStrings;
const
  CSvc = 'IGAUtilsService.GetShopsList';
var
  lRetval: TXDataClientResponse;
  lList: JS.TJSArray;
  lObj: JS.TJSObject;
  I: Integer;
begin
  if FShops = nil then
  begin
    FShops := TStringList.Create;
    lRetval := Await(TXDataClientResponse, WebClient.RawInvokeAsync(CSvc, []));
    lList := JS.ToArray(lRetval.ResultAsObject['value']);
    for I := 0 to lList.Length - 1 do
    begin
      lObj := JS.toObject(lList[I]);
      FShops.AddPair(JS.ToString(lObj.Properties['Name']) + ' (' + JS.ToString(lObj.Properties['Status']) + ')',
        JS.ToString(lObj.Properties['Ref']));
    end;

  end;
  Result := FShops;
end;



end.
